// Pandora implementation of the framework.


#include <pwd.h>
#include <unistd.h>
#include <string>

#include "SDL.h"
#include "SDL_timer.h"
#include "SDL_audio.h"
#include "SDL_video.h"

#include <GLES2/gl2.h>
#include "base/eglport.h"

#include "base/display.h"
#include "base/logging.h"
#include "base/timeutil.h"
#include "gfx_es2/glsl_program.h"
#include "file/zip_read.h"
#include "input/input_state.h"
#include "base/NativeApp.h"
#include "net/resolve.h"
// Simple implementations of System functions



void SystemToast(const char *text) {
	puts(text);
}

void ShowAd(int x, int y, bool center_x) {
	// Ignore ads on Pando
}

void ShowKeyboard() {
	// Irrelevant on Pando
}

void Vibrate(int length_ms) {
	// Ignore on Pando
}

void System_InputBox(const char *title, const char *defaultValue) {
	// Stub
	NativeMessageReceived((std::string("INPUTBOX:") + title).c_str(), "TestFile");
}

void LaunchBrowser(const char *url)
{
	ILOG("Would have gone to %s but LaunchBrowser is not implemented on this platform", url);
}

void LaunchMarket(const char *url)
{
	ILOG("Would have gone to %s but LaunchMarket is not implemented on this platform", url);
}

void LaunchEmail(const char *email_address)
{
	ILOG("Would have opened your email client for %s but LaunchEmail is not implemented on this platform", email_address);
}



const int buttonMappings[14] = {
	SDLK_PAGEDOWN,		//X => cross
	SDLK_END,		//B => circle
	SDLK_HOME,		//A => box
	SDLK_PAGEUP,		//Y => triangle
	SDLK_RSHIFT,		//LBUMPER
	SDLK_RCTRL,		//RBUMPER
	SDLK_LALT,		//START
	SDLK_LCTRL,		//SELECT
	SDLK_UP,		//UP
	SDLK_DOWN,		//DOWN
	SDLK_LEFT,		//LEFT
	SDLK_RIGHT,		//RIGHT
	SDLK_m,			//MENU
	SDLK_BACKSPACE,		//BACK
};

void SimulateGamepad(const uint8 *keys, InputState *input/*, SDL_Joystick *ljoy, SDL_Joystick *rjoy*/) {
	input->pad_buttons = 0;
	input->pad_lstick_x = 0;
	input->pad_lstick_y = 0;
	input->pad_rstick_x = 0;
	input->pad_rstick_y = 0;
	for (int b = 0; b < 14; b++) {
		if (keys[buttonMappings[b]])
			input->pad_buttons |= (1<<b);
	}
/*   There is bug somewhere with joystick+sound+gles on the SDL for Pandora, so no joystick for now...
	if ((ljoy) || (rjoy)) {
		SDL_JoystickUpdate();
		// tolerence is 50%...
		if (ljoy) {
			input->pad_lstick_x = SDL_JoystickGetAxis(ljoy, 0) / 16384;
			input->pad_lstick_y = SDL_JoystickGetAxis(ljoy, 1) / 16384;
		}
		if (rjoy) {
			input->pad_rstick_x = SDL_JoystickGetAxis(rjoy, 0) / 16384;
			input->pad_rstick_y = SDL_JoystickGetAxis(rjoy, 1) / 16384;
		}
	}
*/
// What are those lstick and rstick ? They doesn't exist on the real PSP
// can be mapped to the real rnub and lnub of the Pandora...
/*
	if (keys[SDLK_i])
		input->pad_lstick_y=1;
	else if (keys[SDLK_k])
		input->pad_lstick_y=-1;
	if (keys[SDLK_j])
		input->pad_lstick_x=-1;
	else if (keys[SDLK_l])
		input->pad_lstick_x=1;
	if (keys[SDLK_KP8])
		input->pad_rstick_y=1;
	else if (keys[SDLK_KP2])
		input->pad_rstick_y=-1;
	if (keys[SDLK_KP4])
		input->pad_rstick_x=-1;
	else if (keys[SDLK_KP6])
		input->pad_rstick_x=1;
*/
}

extern void mixaudio(void *userdata, Uint8 *stream, int len) {
	NativeMix((short *)stream, len / 4);
}

int main(int argc, char *argv[]) {
	std::string app_name;
	std::string app_name_nice;


	float zoom = 1.0f;
	float density = 1.0f;
	bool landscape;

	NativeGetAppInfo(&app_name, &app_name_nice, &landscape);
	
	pixel_xres = 800;
	pixel_yres = 480;

	net::Init();

	if (SDL_Init(SDL_INIT_VIDEO /*| SDL_INIT_JOYSTICK*/) < 0) {
		fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
		return 1;
	}
	if (EGL_Open()) {
		fprintf(stderr, "Unable to initialize EGL\n");
		return 1;
	}

	SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8);
	SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8);
	SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8);
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24);
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
	SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, 8);
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
	SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, 1);

	if (SDL_SetVideoMode(pixel_xres, pixel_yres, 0, SDL_SWSURFACE | SDL_FULLSCREEN ) == NULL) {
		fprintf(stderr, "SDL SetVideoMode failed: Unable to create OpenGL screen: %s\n", SDL_GetError());
		SDL_Quit();
		return(2);
	}
	EGL_Init();

	SDL_WM_SetCaption(app_name_nice.c_str(), NULL);

	// Linux
	char path[512];
	const char *the_path = getenv("HOME");
	if (!the_path) {
		struct passwd* pwd = getpwuid(getuid());
		if (pwd)
			the_path = pwd->pw_dir;
	}
	strcpy(path, the_path);
	if (path[strlen(path)-1] != '/')
		strcat(path, "/");
	NativeInit(argc, (const char **)argv, path, "/tmp", "BADCOFFEE");

	dp_xres = (float)pixel_xres * density / zoom;
	dp_yres = (float)pixel_yres * density / zoom;
	pixel_in_dps = (float)pixel_xres / dp_xres;

	NativeInitGraphics();

	float dp_xscale = (float)dp_xres / pixel_xres;
	float dp_yscale = (float)dp_yres / pixel_yres;


	printf("Pixels: %i x %i\n", pixel_xres, pixel_yres);
	printf("Virtual pixels: %i x %i\n", dp_xres, dp_yres);

	SDL_AudioSpec fmt;
	fmt.freq = 44100;
	fmt.format = AUDIO_S16;
	fmt.channels = 2;
	fmt.samples = 1024;
	fmt.callback = &mixaudio;
	fmt.userdata = (void *)0;

	if (SDL_OpenAudio(&fmt, NULL) < 0) {
		ELOG("Failed to open audio: %s", SDL_GetError());
		return 1;
	}

	// Audio must be unpaused _after_ NativeInit()
	SDL_PauseAudio(0);

	InputState input_state;
	int framecount = 0;
	bool nextFrameMD = 0;
	float t = 0;
/*
	SDL_Joystick *ljoy, *rjoy;

	// setup nubs if presents
	ljoy = NULL;
	rjoy = NULL;
	int numjoys=SDL_NumJoysticks();
	if (numjoys>0)
	   for (int i=0; i<numjoys; i++) {
		if (strncmp(SDL_JoystickName(i), "nub0", 4)==0)
			ljoy=SDL_JoystickOpen(i);
		if (strncmp(SDL_JoystickName(i), "nub1", 4)==0)
			rjoy=SDL_JoystickOpen(i);
	}
*/		

	// start main loop
	while (true) {
		input_state.accelerometer_valid = false;
		input_state.mouse_valid = true;
		int quitRequested = 0;

		SDL_Event event;
		while (SDL_PollEvent(&event)) {
			float mx = event.motion.x * dp_xscale;
			float my = event.motion.y * dp_yscale;

			if (event.type == SDL_QUIT) {
				quitRequested = 1;
			} else if (event.type == SDL_KEYDOWN) {
				if (event.key.keysym.sym == SDLK_ESCAPE) {
					quitRequested = 1;
				}
			} else if (event.type == SDL_MOUSEMOTION) {
				input_state.pointer_x[0] = mx;
				input_state.pointer_y[0] = my;
				NativeTouch(0, mx, my, 0, TOUCH_MOVE);
			} else if (event.type == SDL_MOUSEBUTTONDOWN) {
				if (event.button.button == SDL_BUTTON_LEFT) {
					//input_state.mouse_buttons_down = 1;
					input_state.pointer_down[0] = true;
					nextFrameMD = true;
					NativeTouch(0, mx, my, 0, TOUCH_DOWN);
				}
			} else if (event.type == SDL_MOUSEBUTTONUP) {
				if (event.button.button == SDL_BUTTON_LEFT) {
					input_state.pointer_down[0] = false;
					nextFrameMD = false;
					//input_state.mouse_buttons_up = 1;
					NativeTouch(0, mx, my, 0, TOUCH_UP);
				}
			}
		}

		if (quitRequested)
			break;

		const uint8 *keys = (const uint8 *)SDL_GetKeyState(NULL);
		if (keys[SDLK_ESCAPE])
			break;
		SimulateGamepad(keys, &input_state/*, ljoy, rjoy*/);
		UpdateInputState(&input_state);
		NativeUpdate(input_state);
		NativeRender();

		EndInputState(&input_state);

		if (framecount % 60 == 0) {
			// glsl_refresh(); // auto-reloads modified GLSL shaders once per second.
		}

		EGL_SwapBuffers();

		// Simple frame rate limiting
		while (time_now() < t + 1.0f/60.0f) {
			sleep_ms(0);
			time_update();
		}
		time_update();
		t = time_now();
		framecount++;
	}
	// Faster exit, thanks to the OS. Remove this if you want to debug shutdown
	// The speed difference is only really noticable on Linux. On Windows you do notice it though
	NativeShutdownGraphics();
	SDL_PauseAudio(1);
	SDL_CloseAudio();
/*
	if (ljoy)
		SDL_JoystickClose(ljoy);
	if (rjoy)
		SDL_JoystickClose(rjoy);
*/
	NativeShutdown();
	EGL_Close();
	SDL_Quit();
	net::Shutdown();
	exit(0);
	return 0;
}
